#!/usr/bin/perl -w
#
# partmap.pm - partition map module
# by Brian Bresen <bbresen@linuxppc.org>
# under the GPL.
# 
# modified a little by MATSUBAYASHI 'Shaolin' Kohji <shaolin@rhythmaning.org>
#

# This package will allow you to read a drive's partition map
# The partmap object will also contain the block 0 block as well

package partmap;

use strict;
use Fcntl;
use dpme;
my $NULL = 0;

require 'jcode.pl';

sub new {
	my $class = shift @_;
	my $drive = shift @_;
	
	my $map;

	#print "Looking for Signature 4552 in hex, 17746 in decimal\n";
	#my $block0_sig = unpack("B16", pack ("n", 0x4552));

	$map->{name} = $drive;
	$map->{numparts} = 0;
	$map->{disk_order} = $NULL;
	$map->{base_order} = $NULL;
	bless ($map, $class);

	$map->read_block_zero($drive);
# I'll leave it up to coders to fill the map data like below
#	$map->read_partition_map(); 	#read first partmap entry

	return $map;
}

#METHOD: fillGaps - this will place Apple_Free partitions where
# blocks are not part of existing partitions
sub fillGaps {
	my $map = shift;

	my $addCount = 0;	# this is how many gaps we found
	my $index = 1;		# what block we are at with checking
	my ($new_entry,$length,$lastBase,$new_number);

	#get first partition by disk
	my $lastOrder = $map->disk_order;
	#use it to find last by disk
	while (($lastOrder->next_by_disk) != $NULL) {
		$lastOrder = $lastOrder->next_by_disk;
	}

	#get first partition by base
	my $entry = $map->base_order;
	$lastBase = $entry;

	while ($entry != $NULL) {
		if ($index < $entry->dpmePblockStart) { 	#we have a winner
			$new_entry = new dpme;

			#insert the new entry in the right places
			$lastOrder->appendDiskOrder($new_entry);
			$entry->prependBaseOrder($new_entry);
		
			# data fill new_entry
			$new_entry->dpmeSig(20557);

			#calc length for new_entry
			$length = $entry->dpmePblockStart - $index;
			$new_number = $entry->dpmeMapEntry + 1;

			$new_entry->dpmePblockStart($index);
			$new_entry->dpmePblocks($length);
			$new_entry->dpmeLblockStart(0);
			$new_entry->dpmeLblocks($new_entry->dpmePblocks);
			$new_entry->dpmeMapEntry($new_number);
			$map->setPartName($new_entry,"Extra");
			$map->setPartType($new_entry,"Apple_Free");

			# fix numparts, calc new index, and set new last by order
			$map->adjustCount($new_number);
			$index += ($length + $entry->dpmePblocks);
			$lastOrder = $new_entry;
		} else {
			$index += $entry->dpmePblocks;
		}
		$lastBase = $entry;
		$entry = $entry->next_by_base;
	}
print  "index is now    $index\n";
printf "total blocks is %s \n",$map->sbBlkCount;
printf "last by disk name is %s and its base is %s \n",$lastOrder->dpmeName,$lastOrder->dpmePblockStart;
	if ($index < $map->sbBlkCount) {
	    if ((substr(($lastOrder->dpmeType),0,10)) eq "Apple_Free") {
		# maybe disk is formatted with Japanese version of
		# Drive Setup, so dig some fixes here...
		$length = $map->sbBlkCount - $index + 1 + $lastOrder->dpmePblocks;
		$lastOrder->dpmePblocks($length);
		$map->setPartName($lastOrder,"Extra");
	    } else {
		$new_entry = new dpme;

		#insert the new entry in the right places
		$lastOrder->appendDiskOrder($new_entry);
		$lastBase->appendBaseOrder($new_entry);
		
		# data fill new_entry
		$new_entry->dpmeSig(20557);

		#calc length for new_entry
		$length = $map->sbBlkCount - $index + 1;
		$new_number = $lastBase->dpmeMapEntry + 1;

		$new_entry->dpmePblockStart($index);
		$new_entry->dpmePblocks($length);
		$new_entry->dpmeLblockStart(0);
		$new_entry->dpmeLblocks($new_entry->dpmePblocks);
		$new_entry->dpmeMapEntry($new_number);
		$map->setPartName($new_entry,"Extra");
		$map->setPartType($new_entry,"Apple_Free");

		# fix numparts
		$map->adjustCount($new_number);
	    }
	}
}

sub insertPart {
	my $map = shift @_;
	my $entry = shift @_;
	my ($type,$name,$blocks) = @_;

	my $new_free = new dpme;
	#setup new link list order
	$new_free->prev_by_disk($entry);
	$new_free->prev_by_base($entry);
	$new_free->next_by_disk($entry->next_by_disk);
	$new_free->next_by_base($entry->next_by_base);
	$entry->next_by_disk($new_free);
	$entry->next_by_base($new_free);

	#need to setup both entries data
	#setup signature for new_free
	$new_free->dpmeSig(20557);

	#set new entry blocks in map
	my $new_num = ($entry->dpmeMapEntry + 1);
	$new_free->dpmeMapEntry($new_num);
	$entry->dpmeMapEntry($new_num);

	#set new blocks in both and start block in new_free
	$new_free->dpmePblockStart($entry->dpmePblockStart + $blocks);
	$new_free->dpmePblocks($entry->dpmePblocks - $blocks);
	$new_free->dpmeLblockStart(0);
	$new_free->dpmeLblocks($new_free->dpmePblocks);
	$entry->dpmePblocks($blocks);
	$entry->dpmeLblocks($blocks);
	
	#set name and type
	$map->setPartName($entry,$name);
	$map->setPartType($entry,$type);
	$map->setPartName($new_free,"Extra");
	$map->setPartType($new_free,"Apple_Free");

	#reset map blocks (numparts) in each entry
	$map->adjustCount($new_num);

	return;
}

sub getEntryByNum {
	my $map = shift;
	my $part = shift;

	my $entry = $map->disk_order;
	my $i;
	for ($i = 1; $i < $part; $i++) {
		$entry = $entry->next_by_disk;
	}
	return $entry;
}

sub combineFree {
	my $map = shift;
	my $entry = shift;

	my $partsRemoved = 0;
	my $freeType = "Apple_Free";

	my $next = $entry->next_by_base;
	my $prev = $entry->prev_by_base;

	if (($next != $NULL) && ((substr(($next->dpmeType),0,10)) eq $freeType)) {
		$map->combineData($entry,$next);
		$partsRemoved++;
	}

	if (($prev != $NULL) && ((substr(($prev->dpmeType),0,10)) eq $freeType)) {
		$map->combineData($prev,$entry);
		$partsRemoved++;
	}

	print "partsRemoved is: $partsRemoved\n";
	if ($partsRemoved != 0) {
		$map->adjustCount(($entry->dpmeMapEntry) - $partsRemoved);
	}
	return;
}

sub checkChange {
	my $map = shift;

	my $orig_map = new partmap($map->name);
	$orig_map->read_partition_map();

	if (($map->numparts) != ($orig_map->numparts)) {
		return 1;
	}

	my $entry = $map->disk_order;
	my $orig_entry = $orig_map->disk_order;

	while ($entry != $NULL) {
		my $i;
		for $i (keys (%{$entry})) {
			next if ($i eq "dpmeAddrs");
			next if ($i eq "prev_by_base");
			next if ($i eq "next_by_base");
			next if ($i eq "prev_by_disk");
			next if ($i eq "next_by_disk");
			if (($entry->{$i}) ne ($orig_entry->{$i})) {
			printf "Map looks different in %s: %s - %s\n",$i,$entry->{$i},$orig_entry->{$i};
				return 1;
			}
		}
		$entry = $entry->next_by_disk;
		$orig_entry = $orig_entry->next_by_disk;
	}
	return 0;
}

sub combineData {
	my $map = shift;
	my ($first,$second) = @_;

	my $totalBlocks = (($first->dpmePblocks) + ($second->dpmePblocks));
	#Change Data in first
	$first->dpmePblocks($totalBlocks);

	$second->prev_by_base->next_by_base($second->next_by_base);
	if ($second->next_by_base != $NULL) {
		$second->next_by_base->prev_by_base($second->prev_by_base);
	}
	if ($second->prev_by_disk != $NULL) {
		$second->prev_by_disk->next_by_disk($second->next_by_disk);
	}
	if ($second->next_by_disk != $NULL) {
		$second->next_by_disk->prev_by_disk($second->prev_by_disk);
	}
	$second->prev_by_base($NULL);
	$second->next_by_base($NULL);
	$second->prev_by_disk($NULL);
	$second->next_by_disk($NULL);

	#Kill mame and ...
	$second->destroy;

	return;
}

sub adjustCount {
	my $map = shift;
	my $count = shift;

	$map->numparts($count);
	my $entry = $map->disk_order;
	while ($entry != $NULL) {
		$entry->dpmeMapEntry($count);
		$entry = $entry->next_by_disk;
	}
	return;
}


sub disk_report {
	my $map = shift;
	my $drive = $map->name;
	my ($i, $type, $name, $value, $prefix);

	#Output Header
	printf "\nPartition map (with %d byte blocks) on '%s'\n", $map->sbBlkSize, $drive;
	printf " #: %19s %-12s %8s   %-8s ( size )\n", "type", "name", "length", "base";
	#printf " #: %*s %-*s %*s   %-*s ( size )\n", 19, "type", 12, "name", 8, "length", 8, "base";

	#Output Body
	my $count = 1;
	for ($i = $map->disk_order;$i != $NULL;$i = $i->next_by_disk) {
		my @tmparray = split (/\0/, $i->dpmeType);
		$type = $tmparray[0];		
		@tmparray = split (/\0/, $i->dpmeName);
		$name = $tmparray[0];		

		my $isDrvr = $map->containsDriver($count) ? "*" : " ";
		printf "%2.d: %19s%1s%-12.12s ",$count, $type, $isDrvr, $name;
		printf "%8.d @ %-8.d", $i->dpmePblocks, $i->dpmePblockStart;
		$value = ($i->dpmePblocks * $map->sbBlkSize);
		($value,$prefix) = $map->adjust_value_and_compute_prefix($value);
		if (($prefix ne " ") && ($prefix ne "K")) {
			printf " (%5.1f%s)\n", $value, $prefix;
		} else {
			printf "\n";
		}
		$count++;
	}

	#Output Footer
	printf "\n";
	$value = $map->sbBlkCount * $map->sbBlkSize;
	($value,$prefix) = $map->adjust_value_and_compute_prefix($value);

	printf "Device block size=%d, Number of Blocks=%d (%1.1f%1s)\n", $map->sbBlkSize, $map->sbBlkCount, $value, $prefix;
	printf "DeviceType=0x%x, DeviceId=0x%x\n",$map->sbDevType,$map->sbDevId;
	printf "Drivers-\n";

	foreach $i (1 .. $map->sbDrvrCount) {
		printf "%d: @ %d for %d, type=0x%x\n", $i, $map->sbMap($i,'ddBlock'), $map->sbMap($i,'ddSize'), $map->sbMap($i,'ddType');
	}
	printf "\n";
}

sub containsDriver {
	my $map = shift;
	my $part = shift;
	my $i;

	my $entry = $map->disk_order;
	for ($i = $part;$i > 1;$i--) {
		$entry = $entry->next_by_disk;
	}
	my $base = $entry->dpmePblockStart;			# First Block of partition
	foreach $i (1 .. $map->sbDrvrCount ) {
		if ($base == $map->sbMap($i,'ddBlock')) {	# Is same as a driver's first block
			return 1;
		}
	}
	return 0;
}

sub adjust_value_and_compute_prefix {
	my $map = shift;
	my $value = shift || $map;
	my $prefix;

	if ($value < 1024.0) {
		$prefix = " ";
	} else {
		$value = $value / 1024;
		if ($value < 1024.0) {
			$prefix = "K";
		} else {
			$value = $value / 1024;
			if ($value < 1024.0) {
				$prefix = "M";
			} else {
				$value = $value / 1024;
				if ($value < 1024.0) {
						$prefix = "G";
				} else {
					$value = $value / 1024;
					$prefix = "T";
				}
			}
		}
	}
	return ($value,$prefix);
}

sub read_partition_map {
	my $map = shift @_;
	my $drive = $map->name;

	my $temp;

	sysopen(DRIVE, $drive, O_RDONLY)
		or die "Couldn't open $drive for reading!\n"; 

	# Go To the beggining of the drive
	sysseek(DRIVE, 512, 0) 
		or die "Couldn't seek byte 512 of drive!\n";

	#Read the first two bytes from the drive
	$temp = read_16_bits(*DRIVE);

	#print "Read entry sig and it is : $temp\n";

	while ($temp == 20557 ) {
	#HERE WE SHOULD INSERT THE FOLLOWING INTO LINKED LISTS
	#FIRST CREATE NEW NODE THEN CHECK IT'S PLACMENT IN LISTS
	#OR IF IT'S OUT OF RANGE DELETE THE NODE
		my $entry = new dpme;

		$entry->{dpmeSig} = $temp;
		$entry->{dpmeReserved1} = read_16_bits(*DRIVE);
		$entry->{dpmeMapEntry} = read_32_bits(*DRIVE);
		$entry->{dpmePblockStart} = read_32_bits(*DRIVE);
		$entry->{dpmePblocks} = read_32_bits(*DRIVE);

		$entry->{dpmeName} = read_char_32(*DRIVE);	#NUL term'd str
		jcode::convert(\$entry->{dpmeName}, "euc", "sjis");
		$entry->{dpmeType} = read_char_32(*DRIVE);	#1-32 chars

		$entry->{dpmeLblockStart} = read_32_bits(*DRIVE);
		$entry->{dpmeLblocks} = read_32_bits(*DRIVE);
		$entry->{dpmeFlags} = read_32_bits(*DRIVE);
		$entry->{dpmeBootBlock} = read_32_bits(*DRIVE);
		$entry->{dpmeBootBytes} = read_32_bits(*DRIVE);

		$entry->{dpmeAddrs}[1] = read_32_bits(*DRIVE);
		$entry->{dpmeAddrs}[2] = read_32_bits(*DRIVE);
		$entry->{dpmeAddrs}[3] = read_32_bits(*DRIVE);
		$entry->{dpmeAddrs}[4] = read_32_bits(*DRIVE);
		$entry->{dpmeChecksum} = read_32_bits(*DRIVE);

		$entry->{dpmeProcId} = read_char_16(*DRIVE);
		$entry->{dpmeBootArgs} = read_x_bytes(*DRIVE,128);
		$entry->{dpmeReserved3} = read_x_bytes(*DRIVE,248);

		# Here we add to lists 
		if ($map->insertEntryData($entry)) {
			$temp = read_16_bits(*DRIVE);	#get next 2 bytes for while sig check
		} else {
			$temp = 0;						# We are done reading now
		}
	}
	close(DRIVE) 
		or die "Couldn't close drive filehandle!\n";
}

sub insertEntryData {
	my $map = shift;
	my $entry = shift;

#READ REPORT
#print "\nHere what is read:\n";
#$map->displayPart($entry);

	if (($map->disk_order == $NULL) && ($map->base_order == $NULL)) {
		$map->{disk_order} = $entry;
		$map->{base_order} = $entry;
		return $entry;
	}

	#check if partition overflows drive blocks
	my $endBlock = $entry->dpmePblockStart + ($entry->dpmePblocks - 1 );
	if ($endBlock > $map->sbBlkCount) {
		return 0;
	}

	my $i = $map->base_order;

	#check if overlaps any existing partition
	while ($i != $NULL) {
		my $threshold = $i->dpmePblockStart + ($i->dpmePblocks - 1);
		if (($i->next_by_base == $NULL) && ($entry->dpmePblockStart > $threshold)) {
			$i->appendBaseOrder($entry);
			last;
		} elsif (($endBlock < $i->dpmePblockStart )) {
			$i->prependBaseOrder($entry);
			last;
		} elsif ( ($endBlock >= $i->dpmePblockStart) && ($endBlock <= $threshold)) {
			return 0;
		} elsif (($entry->dpmePblockStart >= $i->dpmePblockStart ) && ($entry->dpmePblockStart <= $threshold)) {
			return 0;
		} elsif ($i->next_by_base == $NULL) {
			return 0;
		}
		$i = $i->next_by_base;
	}

	$i = $map->disk_order;
	while ($i->next_by_disk != $NULL) {
		$i = $i->next_by_disk;
	}
	$i->appendDiskOrder($entry);
	
	if (($entry->prev_by_disk != $NULL) || ($entry->next_by_disk != $NULL)) {
		return $entry;
	} else {
		return 0;
	}
}

sub read_block_zero {
	my $map = shift @_;
	my $drive = shift @_;

	sysopen(DRIVE, $drive, O_RDONLY)
		or die "Couldn't open $drive for reading!\n"; 

	# Go To the beggining of the drive
	sysseek(DRIVE,0,0) 
		or die "Couldn't seek to start of drive!\n";

	#Read the first two bytes from the drive
	$map->{sbSig} = read_16_bits(*DRIVE);

	#print "sbSig is $map->{sbSig} - ";
	if ($map->{sbSig} == 17746) { 
		#print "Horray!\n";
	} else { 
		die "A Horrible Horrible error has happend!!!\n"; 
	}

	# Read the next two bytes - the block size of the device 
	# this should most likely be 512
	$map->{sbBlkSize} = read_16_bits(*DRIVE);

	# Read the next four bytes - the block count of the device 
	# this should most likely be be big number
	$map->{sbBlkCount} = read_32_bits(*DRIVE);

	# Read the next two bytes - the device type
	$map->{sbDevType} = read_16_bits(*DRIVE);

	# Read the next two bytes - the device ID
	$map->{sbDevId} = read_16_bits(*DRIVE);

	# Read the next four bytes - the block count of the device 
	$map->{sbData} = read_32_bits(*DRIVE);

	# Read the next two bytes - Driver Count
	$map->{sbDrvrCount} = read_16_bits(*DRIVE);

	# Read the next sbMap upto array[247]
	my $i;

	foreach $i (1 .. $map->{sbDrvrCount}) { 
		$map->{sbMap}[$i]->{ddBlock} = read_32_bits(*DRIVE);
		#print "Driver Starting Block is $map->{sbMap}[$i]->{ddBlock}\n";
		$map->{sbMap}[$i]->{ddSize} = read_16_bits(*DRIVE);
		#print "Driver Size is $map->{sbMap}[$i]->{ddSize}\n";
		$map->{sbMap}[$i]->{ddType} = read_16_bits(*DRIVE);
		#print "Driver Type is $map->{sbMap}[$i]->{ddType}\n";
	}

	close(DRIVE) 
		or die "Couldn't close drive filehandle!\n";

}

sub write_partition_map {
	my $map = shift;

	my $drive = $map->name;
	printf "Writing partition map for %s\n",$drive;

	sysopen(DRIVE, $drive, O_RDWR)
		or die "Couldn't open $drive for reading and writing!\n"; 

	#Go to start of partition map partition
	sysseek(DRIVE, 512, 0) 
		or die "Couldn't seek to start of partition map!\n";

	my $entry = $map->disk_order;
	while ($entry != $NULL) {
		$map->write_partition_map_entry($entry,*DRIVE);
		$entry = $entry->next_by_disk;
	}
	close(DRIVE) 
		or die "Couldn't close drive filehandle!\n";

	return;
}

sub write_partition_map_entry {
	my $map = shift;
	my $entry = shift;
	local (*DRIVE) = shift;
	
	write_16_bits(*DRIVE, $entry->dpmeSig);
	write_16_bits(*DRIVE, $entry->dpmeReserved1);
	write_32_bits(*DRIVE, $entry->dpmeMapEntry);
	write_32_bits(*DRIVE, $entry->dpmePblockStart);
	write_32_bits(*DRIVE, $entry->dpmePblocks);

	jcode::convert(\$entry->{dpmeName}, "sjis", "euc");
	write_char_32(*DRIVE, $entry->dpmeName);
	write_char_32(*DRIVE, $entry->dpmeType);

	write_32_bits(*DRIVE, $entry->dpmeLblockStart);
	write_32_bits(*DRIVE, $entry->dpmeLblocks);
	write_32_bits(*DRIVE, $entry->dpmeFlags);
	write_32_bits(*DRIVE, $entry->dpmeBootBlock);
	write_32_bits(*DRIVE, $entry->dpmeBootBytes);
	
	write_32_bits(*DRIVE, $entry->dpmeAddrs(1));
	write_32_bits(*DRIVE, $entry->dpmeAddrs(2));
	write_32_bits(*DRIVE, $entry->dpmeAddrs(3));
	write_32_bits(*DRIVE, $entry->dpmeAddrs(4));
	write_32_bits(*DRIVE, $entry->dpmeChecksum);

	write_char_16(*DRIVE, $entry->dpmeProcId);
	write_x_bytes(*DRIVE, 128, $entry->dpmeBootArgs);
	write_x_bytes(*DRIVE, 248, $entry->dpmeReserved3);

	return;
}

sub write_16_bits {
	local (*DRIVE) = shift;
	my $temp = shift;

	if (! defined($temp)) {
		$temp = 0;
	}
	my $data = pack("n", $temp);

	my $written =  syswrite(DRIVE, $data, 2, 0);
		die "Fatal error during write!\n" unless defined $written;
	#print "I actually just wrote 2 bytes : $data\n";
	return $data;
}

sub write_32_bits {
	local (*DRIVE) = shift;
	my $temp = shift;
	
	if (! defined($temp)) {
		$temp = 0;
	}
	my $data = pack("N", $temp);
	my $written =  syswrite(DRIVE, $data, 4, 0);
		die "Fatal error during write!\n" unless defined $written;
	#print "I actually just wrote 4 bytes : $data\n";
	return $data;
}

sub write_char_16 {
	local (*DRIVE) = shift;
	my $data = shift;

	if (! defined($data)) {
		$data = "";
	}
	if (length($data) != 16) {
		$data = pack ("a16",$data);
	}

	my $written =  syswrite(DRIVE, $data, 16, 0);
		die "Fatal error during write!\n" unless defined $written;
	#print "I actually just wrote 16 bytes :$data\n";
	return $data;
}

sub write_char_32 {
	local (*DRIVE) = shift;
	my $data = shift;

	if (! defined($data)) {
		$data = "";
	}
	if (length($data) != 32) {
		$data = pack ("a32",$data);
	}

	my $written =  syswrite(DRIVE, $data, 32, 0);
		die "Fatal error during write!\n" unless defined $written;
	#print "I actually just wrote 32 bytes :$data\n";
	return $data;
}

sub write_x_bytes {
	local (*DRIVE) = shift;
	my $bytes = shift;
	my $data = shift;

	if (! defined($data)) {
		$data = "";
	}
	if (length($data) != $bytes) {
		$data = pack ("a$bytes",$data);
	}

	my $written =  syswrite(DRIVE, $data, $bytes, 0);
		die "Fatal error during write!\n" unless defined $written;
	#print "I actually just wrote $bytes bytes :$data\n";
	return $data;
}

sub read_16_bits {
	#my $map = shift;
	local (*DRIVE) = shift;
	my $temp;

	my $read = sysread(DRIVE, $temp, 2, 0);
	#my $temppack = substr("0" x 16 . $temp, -16);
	my $temppack = unpack("B16", $temp);
	return unpack("n", pack("B16", $temppack));
}

sub read_32_bits {
	#my $map = shift;
	local (*DRIVE) = shift;
	my $temp;

	my $read = sysread(DRIVE, $temp, 4, 0);
	#my $temppack = substr("0" x 32 . $temp, -32);
	#guess this is wrong or something dunno 
	my $data = unpack("B32", $temp);
	return unpack("N", pack("B32", $data));
}

sub read_char {
	#my $map = shift;
	local (*DRIVE) = shift;
	my $temp;
	
	my $read = sysread(DRIVE, $temp, 1, 0);
	return $temp;
}

sub read_char_16 {
	#my $map = shift;
	local (*DRIVE) = shift;
	my $temp;

	my $read = sysread(DRIVE, $temp, 16, 0); 
	return $temp;
}

sub read_char_32 {
	#my $map = shift;
	local (*DRIVE) = shift;
	my $temp;

	my $read = sysread(DRIVE, $temp, 32, 0);
	return $temp;
}

sub read_x_bytes {
	#my $map = shift;
	local (*DRIVE) = shift;
	my $bytes = shift;
	my $temp;

	my $read = sysread(DRIVE, $temp, $bytes, 0);
	return $temp;
}

sub displayPart{
	my $map = shift @_;
	my $part = shift @_;

	my ($i,$entry);

	if (ref($part) ne "dpme") {
		$entry = $map->disk_order;
		for ($i = 1; $i < $part; $i++ ) {
			$entry = $entry->next_by_disk;
		}
	} else {
		$entry = $part;
	}
	
	printf "Partiton Sig: %s \n", $entry->dpmeSig;
	printf "Partiton Name: %s \n", $entry->dpmeName;
	printf "Partiton Type: %s \n", $entry->dpmeType;
	printf "Partiton Reservered1: %s \n", $entry->dpmeReserved1;
	printf "Partiton MapEntry: %s \n", $entry->dpmeMapEntry;
	printf "Partiton PblockStart: %s \n", $entry->dpmePblockStart;
	printf "Partiton Pblocks: %s \n", $entry->dpmePblocks;
	printf "Partiton LblockStart: %s \n", $entry->dpmeLblockStart;
	printf "Partiton Lblocks: %s \n", $entry->dpmeLblocks;
	printf "Partiton Flgas: %s \n", $entry->dpmeFlags;
	printf "Partiton BootBlock: %s \n", $entry->dpmeBootBlock;
	printf "Partiton BootBytes: %s \n", $entry->dpmeBootBytes;
	printf "Partiton Addrs[1]: %s \n", $entry->dpmeAddrs(1);
	printf "Partiton Addrs[2]: %s \n", $entry->dpmeAddrs(2);
	printf "Partiton Addrs[3]: %s \n", $entry->dpmeAddrs(3);
	printf "Partiton Addrs[4]: %s \n", $entry->dpmeAddrs(4);
	printf "Partiton Checksum: %s \n", $entry->dpmeChecksum;
	printf "Partiton ProcId: %s \n", $entry->dpmeProcId;
	printf "Partiton BootArgs: %s \n", $entry->dpmeBootArgs;
	printf "Partiton Reservered3: %s \n", $entry->dpmeReserved3;

	printf "This entry node: %s \n",$entry;
	if ($entry->prev_by_disk != $NULL) {
		printf "Previous by disk is: %s \n",$entry->prev_by_disk;
		printf "Next by disk of Previous by disk is: %s \n",$entry->prev_by_disk->next_by_disk;
	}
	if ($entry->next_by_disk != $NULL) {
		printf "Next by disk is: %s \n",$entry->next_by_disk;
		printf "Prev by disk of Next by disk is: %s \n",$entry->next_by_disk->prev_by_disk;
	}
}

sub setPartName{
	my $map = shift;
	my ($entry, $name) = @_;

	if (defined $name) {
		$name = $name . "\0";
	} else {
		return 0;
	}

	my $len = length($name);
	if ($len <= 32) {
		$entry->{dpmeName} = $name;
	} else {
		return 0;
	}
	return $name;
}

sub setPartType{
	my $map = shift;
	my ($entry, $type) = @_;

	if (defined $type) {
		$type = $type . "\0";
	} else {
		return 0;
	} 

	my $len = length($type);
	if ($len <= 32) {
		$entry->{dpmeType} = $type;
	} else {
		return 0;
	}
	return $type;
}

sub disk_order {
	my $self = shift;

	return $self->{disk_order};
}

sub base_order {
	my $self = shift;

	return $self->{base_order};
}

sub sbSig {
	my $self = shift;

	return $self->{sbSig};
}

sub sbBlkSize {
	my $self = shift;

	return $self->{sbBlkSize};
}

sub sbBlkCount {
	my $self = shift;

	return $self->{sbBlkCount};
}

sub sbDevType {
	my $self = shift;

	return $self->{sbDevType};
}

sub sbDevId {
	my $self = shift;

	return $self->{sbDevId};
}

sub sbData {
	my $self = shift;

	return $self->{sbData};
}

sub sbDrvrCount {
	my $self = shift;

	return $self->{sbDrvrCount};
}

sub sbMap {
	my $self = shift;
	my ($i,$field) = @_;

	return $self->{sbMap}[$i]->{$field};
}

sub numparts {
	my $self = shift;

	return $self->{numparts};
}

sub name {
	my $self = shift;

	return $self->{name};
}

1;
