#!/usr/bin/perl -w
#
# perldisk.pl - perl/Gtk disk partitioning utility
# by Brian Bresen <bbresen@linuxppc.org>
# under the GPL.
#
# modified for Japanese environment
# by MATSUBAYASHI 'Shaolin' Kohji <shaolin@rhythmaning.org>
#
#    LINUXPPC INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE 
#    INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#    FOR A PARTICULAR PURPOSE. 
#
#    IN NO EVENT SHALL LINUXPPC INC. BE LIABLE FOR ANY SPECIAL, INDIRECT, OR 
#    CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM 
#    LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT, 
#    NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION 
#    WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
#
# In short this is powerful stuff and you have been warned.
#
# For real version need to change driveList return and BEGIN block...
#

BEGIN { unshift(@INC, "/usr/X11R6/lib/perldisk"); }

use strict;
use partmap;
use Gtk;

require 'GTK_standard.pl';

my $VERSION = 1.51;		# The version number
my $changesmade = 0;	# Flag set if changes made to map
my $NULL = 0;
my @drive;			# List of devices on machine or params passed
my $map;			#The Currently viewed partiton map object

my $CurrentFont;
my %words;              # hash of words for internationalization


if (defined $ARGV[0]) {
	@drive = @ARGV;
} else {
	@drive = driveList();
}

init Gtk;

if ( -f "/tmp/English" ) {
	make_word_hash( "English" );
} else {
	make_word_hash( "Japanese" );
}

if ( -f "/tmp/English" ) {
	change_font();
} else {
	change_font( "-alias-fixed-medium-r-normal-*-12-*-*-*-*-*-*-*" );
}

# set Japanese locale
if (! -f "/tmp/English" ) {
	$ENV{'LANG'} = "ja_JP.ujis";
	$ENV{'LANGUAGE'} = "ja";
	Gtk->set_locale();
}

create_main_window(@drive);

main Gtk;

sub change_font {
	my( $StandardText, $FontSelector );
	$StandardText = new Gtk::Text(undef,undef);
	$FontSelector  = new Gtk::FontSelectionDialog "Font Selection Dialog";
	my $newFont = $_[0];
	if( ! defined $newFont ) {
		print "Setting font to lucida medium\n";
		# $FontSelector->set_font_name( "-misc-fixed-medium-r-semicondensed-*-13-*-*-*-c-*-iso8859-8" );
		# $FontSelector->set_font_name( "-b&h-lucida-medium-r-normal-sans-12-*-72-72-p-*-*-1" );
		$FontSelector->set_font_name( "-b&h-lucida-medium-r-normal-sans-12-120-75-75-p-71-iso8859-1" );
	} else {
		print "Setting font to $newFont\n";
		$FontSelector->set_font_name( $newFont );
	}
	$CurrentFont = $FontSelector->get_font;
	# print "got font $CurrentFont\n";
	$StandardText->style->font($CurrentFont);
	# print "set font to $CurrentFont\n";
}

sub make_word_hash {
	my( $entry, $key, @stuff );
	print "start make_word_hash\n";
	if( -e "/usr/X11R6/lib/perldisk/$_[0]" ) {
		open( WORDS , "/usr/X11R6/lib/perldisk/$_[0]" );
	} elsif ( -e "/live/perldisk/$_[0]" ) {
		open( WORDS , "/live/perldisk/$_[0]" );
	}
	while( <WORDS> ) {
		chop $_;
		if( $_ eq "" ) { next; };
		@stuff = split( /	/ , $_ );
		if( defined $stuff[1] ) {
			$words{ $stuff[0] } = pop( @stuff );
			next;
		}
		$entry = $_;
		undef $words{ $entry };
		while( <WORDS> ) {
			chop $_;
			if( $_ eq "end" ) { 
				chop $words{ $entry };
				last;
			}
			$words{ $entry } .= "$_\n";
#			print "$_\n";
		}
	}
	close( WORDS );

#	foreach $key ( keys( %words ) ) { print "$key\n"; }
}

sub create_main_window {
	my @drive = (@_);
	my $i;
	my @allmaps;

	# read in the partition maps
	for $i (0 .. $#drive) {
		$allmaps[$i] = new partmap($drive[$i]);
	}
	$map = $allmaps[0];
	$map->read_partition_map();

	# check for invalid map - not all blocks defined
	$map->fillGaps();

	# Main Window
	my $mainWindow = standard_window( 25,25,-1,-1, "PerlDisk - V $VERSION");
	$mainWindow->signal_connect("destroy" => \&Quit);

	# main box to hold all things - it will be vertical
	my $mainVbox = standard_vbox( 5, $mainWindow );

	# Partitions Box - 
	my $partVbox = standard_vbox( 5, $mainVbox, 0, 1, 0 );
	my $partLabel = standard_label( "$words{WARNING1}", $partVbox, 0, 1, 0 );
	$partLabel = standard_label( "$words{Partitions}", $partVbox, 0, 1, 0 );

	# The Partition Entries Each in a clist
	my $partEntryBox = standard_vbox( 2, $partVbox, 0, 0, 0 );
	
	my @titles = ("#","$words{Type}","$words{Name}","$words{Blocks}","$words{Base}","$words{Size}");
	my $partEntryList = new_with_titles Gtk::CList(@titles);

	$partEntryList->set_column_width(0,40);
	$partEntryList->set_column_width(1,115);
	$partEntryList->set_column_width(2,105);
	$partEntryList->set_column_width(3,60);
	$partEntryList->set_column_width(4,60);
	$partEntryList->set_column_width(5,40);

	$partEntryList->set_column_justification(0,"center");
	$partEntryList->set_column_justification(1,"right");
	$partEntryList->set_column_justification(3,"right");
	$partEntryList->set_column_justification(5,"center");

	# Fill partition map data
	fillPartData($map,$partEntryList);

	$partVbox->pack_start($partEntryList, 1, 1, 10);

	$partEntryList->column_titles_show;
	$partEntryList->show;

	#Partition Button Box
	my $partButtonBox = standard_hbox( 2, $partVbox, 0, 1, 0 );

	my $buttonTable = new Gtk::Table(2,7,0);
	$partButtonBox->pack_start($buttonTable, 1, 1, 0);
	$buttonTable->show;

	# The Partition Buttons
	my $partAddButton = table_button("$words{Add}", [\&partAdd, \$map, $partEntryList]);
	$buttonTable->attach($partAddButton, 0, 1, 0, 1, [-fill,-expand], [], 5, 0);
	$partAddButton->show;

	my $nameEntry = new Gtk::Combo();
	$nameEntry->entry->set_text("/");
	$nameEntry->set_popdown_strings('/','swap','/boot','/home','/usr','/var');
	$nameEntry->set_usize(70,0);
	
	my $partNameButton = table_button("$words{Change_Name}", [\&partName, \$map, $partEntryList, $nameEntry->entry]);

	$buttonTable->attach($partNameButton, 1, 2, 0, 1, [-fill,-expand], [], 5, 0);
	$buttonTable->attach($nameEntry, 1, 2, 1, 2, [-fill,-expand], [], 5, 0);
	$partNameButton->show;
	$nameEntry->show;

	my $partConvertButton = table_button("$words{Convert_Type}", [\&partConvert, \$map, $partEntryList]);
	$buttonTable->attach($partConvertButton, 2, 3, 0, 1, [-fill,-expand], [], 5, 0);
	$partConvertButton->show;

	my $partDeleteButton = table_button("$words{Delete}", [\&partDelete, \$map, $partEntryList]);
	$buttonTable->attach($partDeleteButton, 3, 4, 0, 1, [-fill,-expand], [], 5, 0);
	$partDeleteButton->show;

	my $AboutButton = table_button("$words{About}", \&aboutDialog);
	$buttonTable->attach($AboutButton, 4, 5, 0, 1, [-fill], [], 5, 0);
	$AboutButton->show;

	my $HelpButton = table_button("$words{Help}", \&helpDialog);
	$buttonTable->attach($HelpButton, 5, 6, 0, 1, [-fill], [], 5, 0);
	$HelpButton->show;

	my $QuitButton = table_button("$words{Quit}", \&Quit);
	$buttonTable->attach($QuitButton, 6, 7, 0, 1, [-fill], [], 5, 0);
	$QuitButton->show;

	#Drive Summary Box
	my $driveVbox = standard_vbox( 5, $mainVbox, 0, 0, 0 );

	my $driveLabel = standard_label( "Drive Info:", $driveVbox, 0, 1, 0 );

	@titles = ("Drive","Total Size", "Free", "Used");
	my $driveEntryList = new_with_titles Gtk::CList(@titles);

	# This is for alignment
	foreach $i (0 .. 4) {
		$driveEntryList->set_column_width($i,100);
		$driveEntryList->set_column_justification($i,"center");
	}

	foreach $i (@allmaps) {
		my $blocksTotal = $i->sbBlkCount;
		my $blocksFree = freeBlockCount($i);
		my $percent;
		if ($i->disk_order == $NULL) { 
			$percent = "-"; 
		} else {
			$percent = (($blocksTotal - $blocksFree) / $blocksTotal) * 100;
			$percent = sprintf "%3.1f",$percent;
			$percent .= "%";
		}
		my @tmpentry = ($i->name, $blocksTotal, $blocksFree, $percent);
		$driveEntryList->append(@tmpentry);
	}
	$driveEntryList->select_row(0,0);		#Highlight initial drive
	$driveEntryList->signal_connect('select_row', [\&selectDrive, $partEntryList, @allmaps] );

	$driveVbox->pack_start($driveEntryList, 1, 1, 10);

	$driveEntryList->column_titles_show;
	$driveEntryList->show;
}

sub addPrompt {
	my $size = shift @_;

	my ($name,$blocks);
	my $type = "Apple_UNIX_SVR2";
	my ($value,$suffix);

	my $addWin = new Gtk::Dialog();
	$addWin->set_uposition( 100, 100);
	$addWin->signal_connect("destroy", \&destroy_window, \$addWin);
	$addWin->signal_connect("delete_event", \&destroy_window, \$addWin);
	$addWin->set_title("Select new partition parameters");
	$addWin->show;
	$addWin->set_modal(1);

	my $adjustment = new Gtk::Adjustment($size, 0, $size, 1.0, 2048.0, 0.0);

	#add dial for blocks
	my $addDial = new Gtk::Dial $adjustment;
	$addDial->set_update_policy(-continuous);
	$addWin->vbox->pack_start($addDial, 0, 0, 0);
	$addDial->show;

	#add horiz box for spinbutton and size suffix
	my $addEntryBox = standard_hbox( 2, $addWin->vbox, 0, 1, 0 );

	#add spinbutton to show block value
	($value,$suffix) = getBlockString($size);

	my $addEntry = new Gtk::SpinButton ($adjustment, 1, 0);
	$addEntry->set_value($size);
	$addEntryBox->pack_start($addEntry, 1, 1, 0);
	$addEntry->show;

	$addEntry->signal_connect('key_press_event', \&blockKeyPress);
	$addEntry->signal_connect_after('key_press_event', \&blockKeyPressAfter);

	my $addSuffixLabel = new Gtk::Label($suffix);
	$addEntryBox->pack_start($addSuffixLabel, 0, 1, 0);
	$addSuffixLabel->show;

	my $addText = "$words{ADD1}\n$words{ADD2}";
	my $addLabel = new Gtk::Label $addText;
	$addWin->vbox->pack_start($addLabel, 0, 0, 0);
	$addLabel->show;

	#add menu button for type
	# not just yet -- TODO

	#add combo widget for name
	my $nameEntry = new Gtk::Combo();
	$nameEntry->entry->set_text("/");
	$nameEntry->set_popdown_strings('/','swap','/boot','/home','/usr','/var');
	$nameEntry->set_usize(70,0);
	$addWin->vbox->pack_start($nameEntry, 0, 0, 0);
	$nameEntry->show;

	my $addOk = new Gtk::Button "$words{Add}";
	$addOk->signal_connect("clicked", sub {
				$blocks = $addDial->get_adjustment->value; 
				$name = $nameEntry->entry->get_text;
				$addWin->destroy;
	});
	$addWin->action_area->pack_start($addOk, 1, 1, 0);
	$addOk->show;

	my $addDont = new Gtk::Button "$words{Cancel}";
	$addDont->signal_connect("clicked", sub {
				$blocks = 0; 
				$name = "Anything";
				$addWin->destroy;
	});
	$addWin->action_area->pack_start($addDont, 1, 1, 0);
	$addDont->show;

	while (! defined $name) {
		my $newval = $addEntry->get_value_as_int();
#		$addDial->set_value($newval);
		($value,$suffix) = getBlockString($newval);
		$addSuffixLabel->set($suffix);

		#Keep updating main window during modalhood
		main_iteration Gtk;
	}
	return ($type,$name,int($blocks));
}

sub blockKeyPress {
	my ($addEntry, $event) = @_;
	my $lastchar = chr($event->{'keyval'});
	my $newsize = $addEntry->get_value_as_int;
	if (($lastchar eq "G") || ($lastchar eq "g")) {
		$newsize = ($newsize * 1024 * 2048);
		$addEntry->set_value($newsize);
		main_iteration Gtk;
	} elsif (($lastchar eq "M") || ($lastchar eq "m")) {
		$newsize = ($newsize * 2048);
		$addEntry->set_value($newsize);
		main_iteration Gtk;
	} elsif ($event->{'keyval'} == 65288) {		#this is for the backspace key
		my $frontspace = $addEntry->get_value_as_int;
		chop $frontspace;
		if ($frontspace eq "") { 
			$frontspace = 0;
		}
		$addEntry->set_value($frontspace);
		main_iteration Gtk;
	} elsif ($event->{'keyval'} == 65535) {		#this is for the delete key
		my $frontdelete = $addEntry->get_value_as_int;
		chop $frontdelete;
		if ($frontdelete eq "") { 
			$frontdelete = 0;
		}
		$addEntry->set_value($frontdelete);
		main_iteration Gtk;
	} elsif (($event->{'keyval'} >= 65456) && ($event->{'keyval'} <= 65465)) {
		my $currentVal = $addEntry->get_value_as_int;
		my $inputNum = $event->{'keyval'} - 65456; 
		$addEntry->set_value($currentVal . $inputNum);
		main_iteration Gtk;
	} elsif ($lastchar =~ /[0-9]/) {
		my $currentVal = $addEntry->get_value_as_int;
		$addEntry->set_value($currentVal . chr($event->{'keyval'}));
		main_iteration Gtk;
	}
}

sub blockKeyPressAfter { 
	my ($addEntry, $event) = @_;
	my $newsize = $addEntry->get_value_as_int;
	$addEntry->set_value($newsize);
	#	method spinbutton->update is not in pre-0.7000 Gtk-perl
	#	so hack around it 
	#$addEntry->update;
	main_iteration Gtk;
}

sub freeBlockCount {
	my $map = shift @_;
	my $free = 0;

	if ($map->disk_order == $NULL) {
		return "-";
	}
	my $i;
	for ($i = $map->disk_order;$i != $NULL; $i = $i->next_by_disk) {
		my $tmpname = $i->dpmeType;
		if (substr($tmpname,0,10) eq "Apple_Free") { 
			$free += $i->dpmePblocks;
		}
	}
	return $free;
}

sub rebootPrompt {
	my $choice;
	
	my $rebootWin = new Gtk::Dialog();
	$rebootWin->set_uposition( 100, 100);
	$rebootWin->signal_connect("destroy", \&destroy_window, \$rebootWin);
	$rebootWin->signal_connect("delete_event", \&destroy_window, \$rebootWin);
	$rebootWin->set_title("Reboot?");
	$rebootWin->show;
	$rebootWin->grab_focus();
	$rebootWin->set_modal(1);

	my $rebootText = "$words{REBOOT1}\n$words{REBOOT2}\n$words{REBOOT3}";
	my $rebootLabel = new Gtk::Label $rebootText;
	$rebootWin->vbox->pack_start($rebootLabel, 0, 0, 0);
	$rebootLabel->show;

	my $rebootOk = new Gtk::Button "$words{Reboot}";
	$rebootOk->signal_connect("clicked", sub {$choice = 1; $rebootWin->destroy;});
	$rebootWin->action_area->pack_start($rebootOk, 1, 1, 0);
	$rebootOk->show;

	my $rebootDont = new Gtk::Button "$words{NoReboot}";
	$rebootDont->signal_connect("clicked", sub {$choice = 0; $rebootWin->destroy;});
	$rebootWin->action_area->pack_start($rebootDont, 1, 1, 0);
	$rebootDont->show;

	while (! defined $choice) {
		main_iteration Gtk;
	}
	return $choice;
}

sub savePrompt {
	my $map = shift;

	my $choice;

	my $saveWin = new Gtk::Dialog();
	$saveWin->set_uposition( 50, 50);
	$saveWin->signal_connect("destroy", \&destroy_window, \$saveWin);
	$saveWin->signal_connect("delete_event", \&destroy_window, \$saveWin);
	$saveWin->set_title("Save changes?");
	$saveWin->show;
	$saveWin->grab_focus();
	$saveWin->set_modal(1);

	my $saveText = "$words{SAVETEXT1} \n$words{SAVETEXT2}";
	$saveText .= "\n\n$words{SAVETEXT3}\n$words{SAVETEXT4}\n";
$saveText .= "\n\n";
$saveText .= "LINUXPPC INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE \n";
$saveText .= "    INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS \n";
$saveText .= "    FOR A PARTICULAR PURPOSE. \n";
$saveText .= "\n";
$saveText .= "    IN NO EVENT SHALL LINUXPPC INC. BE LIABLE FOR ANY SPECIAL, INDIRECT, OR \n";
$saveText .= "    CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM \n";
$saveText .= "    LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT, \n";
$saveText .= "    NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION \n";
$saveText .= "    WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. \n";
$saveText .= "\n";
$saveText .= " $words{SAVETEXT13}\n";
$saveText .= "\n";
	my $saveLabel = new Gtk::Label $saveText;
	$saveWin->vbox->pack_start($saveLabel, 0, 0, 0);
	$saveLabel->show;

	my $saveOk = new Gtk::Button "$words{Destructive_Save}";
	$saveOk->signal_connect("clicked", sub {$choice = 1; $saveWin->destroy;});
	$saveWin->action_area->pack_start($saveOk, 1, 1, 0);
	$saveOk->show;

	my $saveDont = new Gtk::Button "$words{Safely_Exit}";
	$saveDont->signal_connect("clicked", sub {$choice = 0; $saveWin->destroy;});
	$saveWin->action_area->pack_start($saveDont, 1, 1, 0);
	$saveDont->show;

	while (! defined $choice) {
		main_iteration Gtk;
	}
	return $choice;
}

sub helpDialog {
	my $choice;

	my $helpWin = new Gtk::Dialog();
	$helpWin->set_uposition(50,50);
	$helpWin->signal_connect("destroy", \&destroy_window, \$helpWin);
	$helpWin->signal_connect("delete_event", \&destroy_window, \$helpWin);
	$helpWin->set_title("PerlDisk Help");
	$helpWin->show;
	$helpWin->grab_focus();
	$helpWin->set_modal(1);

	my $helpText = "perldisk.pl - perl/Gtk disk partitioning utility\n";
	$helpText .= "by Brian Bresen <bbresen\@linuxppc.org>\n";
	$helpText .= "under the GPL.\n\n";
	$helpText .= "$words{HELP1}\n";
	$helpText .= "$words{HELP2}\n";
	$helpText .= "$words{HELP3}\n";
	$helpText .= "$words{HELP4}\n";
	$helpText .= "$words{HELP5}\n";
	$helpText .= "$words{HELP6}\n";
	$helpText .= "$words{HELP7}\n";
	$helpText .= "$words{HELP8}\n";
	$helpText .= "\n";

	my $helpLabel = new Gtk::Label $helpText;
	$helpWin->vbox->pack_start($helpLabel, 0, 0, 0);
	$helpLabel->show;

	my $helpOk = new Gtk::Button "OK";
	$helpOk->signal_connect("clicked", sub {$choice = 1; $helpWin->destroy;});
	$helpWin->action_area->pack_start($helpOk, 1, 1, 0);
	$helpOk->show;

	while (! defined $choice) {
		main_iteration Gtk;
	}
	return $choice;
}

sub aboutDialog {
	my $choice;

	my $aboutWin = new Gtk::Dialog();
	$aboutWin->set_uposition( 50, 50);
	$aboutWin->signal_connect("destroy", \&destroy_window, \$aboutWin);
	$aboutWin->signal_connect("delete_event", \&destroy_window, \$aboutWin);
	$aboutWin->set_title("About Author");
	$aboutWin->show;
	$aboutWin->grab_focus();
	$aboutWin->set_modal(1);

	my $aboutText = "perldisk.pl - perl/Gtk disk partitioning utility\n";
	$aboutText .= "by Brian Bresen <bbresen\@linuxppc.org>\n";
	$aboutText .= "under the GPL.\n";
	$aboutText .= "\n\n";
	$aboutText .= "LINUXPPC INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE \n";
	$aboutText .= "    INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS \n";
	$aboutText .= "    FOR A PARTICULAR PURPOSE. \n\n";
	$aboutText .= "    IN NO EVENT SHALL LINUXPPC INC. BE LIABLE FOR ANY SPECIAL, INDIRECT, OR \n";
	$aboutText .= "    CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM \n";
	$aboutText .= "    LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT, \n";
	$aboutText .= "    NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION \n";
	$aboutText .= "    WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. \n\n";
	$aboutText .= " In short this is powerful stuff and you have been warned.\n";
	$aboutText .= "\n";
	my $aboutLabel = new Gtk::Label $aboutText;
	$aboutWin->vbox->pack_start($aboutLabel, 0, 0, 0);
	$aboutLabel->show;

	my $aboutOk = new Gtk::Button "OK";
	$aboutOk->signal_connect("clicked", sub {$choice = 1; $aboutWin->destroy;});
	$aboutWin->action_area->pack_start($aboutOk, 1, 1, 0);
	$aboutOk->show;

	while (! defined $choice) {
		main_iteration Gtk;
	}
	return $choice;
}

sub selectDrive {
	my ($drivelist) = shift @_;
	my ($partlist) = shift @_;
#	my ($map) = shift @_;
	my @maps = (@_);

	if (defined ($drivelist->selection())) {
		my $row = $drivelist->selection();

		#Check for change in previous map
		if ($map->checkChange()) {
			if (savePrompt($map) ) {
				$map->write_partition_map();
				$changesmade = 1;
			} else {
				my $i = 0;
				while ($maps[$i] != $map) {
					$i++;
				}
				$maps[$i] = $NULL;
			}
		}

		$map = $maps[$row];

		$partlist->clear;
		if ($map->disk_order == $NULL) {
			$map->read_partition_map();

			my $blocksTotal = $map->sbBlkCount;
			my $blocksFree = freeBlockCount($map);
			my $percent = (($blocksTotal - $blocksFree) / $blocksTotal) * 100;
			$percent = sprintf "%3.1f",$percent;
			$percent .= "%";

			$drivelist->set_text($row,2,$blocksFree);
			$drivelist->set_text($row,3,$percent);
		}
		fillPartData($map,$partlist);

		return $map;
	} else {
		print "Strange happenings select_row signal when nothing selected\n";
		return;
	}
}

sub fillPartData {
	my ($map,$clist) = @_;
	my ($i,$value,$prefix);

	my $count = 1;
	for ($i = $map->disk_order;$i != $NULL; $i = $i->next_by_disk) {
		$value = ($i->dpmePblocks * $map->sbBlkSize);
		($value,$prefix) = $map->adjust_value_and_compute_prefix($value);
		if (($prefix ne " ") && ($prefix ne "K")) { 
			$value = sprintf "%5.1f%s",$value,$prefix;
		} else {
			$value = " "; 
		}

		my @tmparray = ($count, $i->dpmeType, $i->dpmeName, $i->dpmePblocks, $i->dpmePblockStart, $value);

		#now append tmparray to the clist 
		$clist->append(@tmparray);

		$count++;
	}
}

sub Quit {
	my $callingWidget = shift @_;

	#Check for change in previous map
	if ($map->checkChange()) {
		if (savePrompt($map) ) {
			$map->write_partition_map();
			$changesmade = 1;
		}
	}
	if (($changesmade) && (rebootPrompt())) {
		system ("init 6");
	} else {
		Gtk->exit(0);
	}
}

sub getBlockString {
	my $blocks = shift;

	my ($value,$prefix);
	($value,$prefix) = partmap::adjust_value_and_compute_prefix($blocks * 512);
	if (($prefix ne " ") && ($prefix ne "K")) { 
		$value = sprintf "%5.1f%s", $value, $prefix;
	} else {
		$value = " ";
	}
	return ($blocks, $value);
}

sub partAdd {
	my ($button) = shift @_;
	my ($map, $list) = @_;

	my $part;

	if (defined ($list->selection())) {
		 $part = ($list->selection() + 1);
	} else {
		print "You need to select a free partiton first\n";
		return;
	}
	my $entry = $$map->getEntryByNum($part);

	my $myName = $entry->dpmeType;
	if ((substr($myName,0,10)) eq "Apple_Free") {
		my $size = $entry->dpmePblocks;
		my ($type, $name, $count) = addPrompt($size);
		#printf "my type: %s, my name: %s, my size: %s\n",$type,$name,$count;
		if ($count != 0) {
			if ($size == $count) {
				$$map->setPartType($entry,$type);
				$$map->setPartName($entry,$name);
			} else {
				$$map->insertPart($entry,$type,$name,$count);
			}
			$list->clear;
			fillPartData($$map,$list);
		}
	} else {
		print "Hey you need to select a free partiton!\n";
		return;
	}
	return;
	
}

sub partDelete {
	my ($button) = shift @_;
	my ($map,$list) = @_;

	my $type = "Apple_Free";
	my $name = "Extra";
	my $part;
	if (defined ($list->selection())) {
		 $part = ($list->selection() + 1);
	} else {
		return;
	}

	my $entry = $$map->getEntryByNum($part);
	#print "Attempting to change type of partition number $part to $type.\n";
	#print "Attempting to change name of partition number $part to $name.\n";

	$$map->setPartType($entry,$type);
	$$map->setPartName($entry,$name);

	$$map->combineFree($entry);

	#Reshow the partmap
	$list->clear;
	fillPartData($$map,$list);

	return;
}

sub partName {
	my ($button) = shift @_;
	my ($map,$list,$combo) = @_;

	my $name = $combo->get_text;
	my $part;
	if (defined ($list->selection())) {
		 $part = ($list->selection() + 1);
	} else {
		return;
	}
	print "Attempting to change name of partition number $part to $name \n";
	my $entry = $$map->getEntryByNum($part);

	$$map->setPartName($entry,$name);
	$part--;						# This is the row again
	$list->set_text($part,2,$name);
}

sub driveList {
	my @drive;

	#$drive[0] = './my_little_drive';
	#$drive[1] = './ide_drive';

	my (@longlist,@list);
	my ($i,$input);

	open (PARTS, "/proc/partitions");

	while ($input = <PARTS>) {
		chop($input);
		next if ($input =~ /name/);
		next if ($input eq "");
		my @temp = split (/ /, $input);
		push (@longlist,(pop(@temp)));
	}
	close(PARTS);

	for $i (0 .. ($#longlist - 1)) {
		if ((length($longlist[$i]) == 3) && (length($longlist[$i+1]) != 3)) {
			push(@list,( "/dev/" . $longlist[$i]));
		}
	}
	return @list;
	#return @drive;
}

sub partConvert {
	my ($button) = shift @_;
	my ($map,$list) = @_;

	my $type = "Apple_UNIX_SVR2";
	my $othertype = "Apple_HFS";
	my $freetype = "Apple_Free";
	my $part;
	my $new_type = "";

	if (defined ($list->selection())) {
		 $part = ($list->selection() + 1);
	} else {
		return;
	}

	# Get parttion map entry data and then the type of filesystem
	my $entry = $$map->getEntryByNum($part);
	my $j = $entry->dpmeType;

	if (substr($j,0,9) eq $othertype) { 
		$new_type = "Apple_UNIX_SVR2";
	}
	if (substr($j,0,10) eq $freetype) { 
		$new_type = "Apple_UNIX_SVR2";
	}
	if (substr($j,0,15) eq $type) { 
		$new_type = "Linux_PPC";
	}
	if (substr($j,0,9) eq "Linux_PPC" ) { 
		$new_type = "Apple_HFS";
	}
	if ( $new_type eq "" ) {
		print "Will only convert HFS, UNIX\(Linux\) or Free Partitions!!\n";
		return;
	}
	print "Attempting to change partition number $part type to $new_type.\n";

	if ($new_type eq "Apple_HFS") {
		$entry->dpmeFlags(1073742719);
	}
	$$map->setPartType($entry,$new_type);
	$part--;						# This is the row again
	$list->set_text($part,1,$new_type);
}
