#!/usr/bin/perl -w
#
# perldisk.pl - perl/Gtk disk partitioning utility
# by Brian Bresen <bbresen@linuxppc.org>
# under the GPL.
#
#    LINUXPPC INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE 
#    INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#    FOR A PARTICULAR PURPOSE. 
#
#    IN NO EVENT SHALL LINUXPPC INC. BE LIABLE FOR ANY SPECIAL, INDIRECT, OR 
#    CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM 
#    LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT, 
#    NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION 
#    WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
#
# In short this is powerful stuff and you have been warned.
#
# For real version need to change driveList return and BEGIN block...
#

BEGIN { unshift(@INC, "/usr/lib/perldisk"); }

use strict;
use partmap;
use Gtk;

require 'GTK_standard.pl';

my $VERSION = 1.3;		# The version number
my $changesmade = 0;	# Flag set if changes made to map
my $NULL = 0;
my @drive;			# List of devices on machine or params passed
my $map;			#The Currently viewed partiton map object

if (defined $ARGV[0]) {
	@drive = @ARGV;
} else {
	@drive = driveList();
}

init Gtk;
create_main_window(@drive);

main Gtk;

sub create_main_window {
	my @drive = (@_);
	my $i;
	my @allmaps;

	# read in the partition maps
	for $i (0 .. $#drive) {
		$allmaps[$i] = new partmap($drive[$i]);
	}
	$map = $allmaps[0];
	$map->read_partition_map();

	# check for invalid map - not all blocks defined
	$map->fillGaps();

	# Main Window
	my $mainWindow = standard_window( 25,25,-1,-1, "Select Partition Dialog");
	$mainWindow->signal_connect("destroy" => \&Quit);

	# main box to hold all things - it will be vertical
	my $mainVbox = standard_vbox( 5, $mainWindow );

	# Partitions Box - 
	my $partVbox = standard_vbox( 5, $mainVbox, 0, 1, 0 );
	my $partLabel = standard_label( "Select a Partition:", $partVbox, 0, 1, 0 );

	# The Partition Entries Each in a clist
	my $partEntryBox = standard_vbox( 2, $partVbox, 0, 0, 0 );
	
	my @titles = ("#","Type","Name","Blocks","Base","Size");
	my $partEntryList = new_with_titles Gtk::CList(@titles);

	$partEntryList->set_column_width(0,40);
	$partEntryList->set_column_width(1,115);
	$partEntryList->set_column_width(2,105);
	$partEntryList->set_column_width(3,60);
	$partEntryList->set_column_width(4,60);
	$partEntryList->set_column_width(5,40);

	$partEntryList->set_column_justification(0,"center");
	$partEntryList->set_column_justification(1,"right");
	$partEntryList->set_column_justification(3,"right");
	$partEntryList->set_column_justification(5,"center");

	# Fill partition map data
	fillPartData($map,$partEntryList);

	$partVbox->pack_start($partEntryList, 1, 1, 10);

	$partEntryList->column_titles_show;
	$partEntryList->show;

	#Partition Button Box
	my $partButtonBox = standard_hbox( 2, $partVbox, 0, 1, 0 );

	my $buttonTable = new Gtk::Table(2,5,0);
	$partButtonBox->pack_start($buttonTable, 1, 1, 0);
	$buttonTable->show;

	my $partConvertButton = table_button("Select", 1, 1, 10, [\&partConvert, \$map, $partEntryList]);
	$buttonTable->attach($partConvertButton, 2, 3, 0, 1, [-fill,-expand], [], 10, 0);
	$partConvertButton->show;

	my $CancelButton = table_button("Cancel", 1, 1, 10, sub {
		Gtk->exit(0);
	} );
	$buttonTable->attach($CancelButton, 5, 6, 0, 1, [-fill,-expand], [], 10, 0);
	$CancelButton->show;

	#Drive Summary Box
	my $driveVbox = standard_vbox( 5, $mainVbox, 0, 0, 0 );

	my $driveLabel = standard_label( "Drive Info:", $driveVbox, 0, 1, 0 );

	@titles = ("Drive","Total Size", "Free", "Used");
	my $driveEntryList = new_with_titles Gtk::CList(@titles);

	# This is for alignment
	foreach $i (0 .. 4) {
		$driveEntryList->set_column_width($i,100);
		$driveEntryList->set_column_justification($i,"center");
	}

	foreach $i (@allmaps) {
		my $blocksTotal = $i->sbBlkCount;
		my $blocksFree = blockCount($i);
		my $percent;
		if ($i->disk_order == $NULL) { 
			$percent = "-"; 
		} else {
			$percent = (($blocksTotal - $blocksFree) / $blocksTotal) * 100;
			$percent = sprintf "%3.1f",$percent;
			$percent .= "%";
		}
		my @tmpentry = ($i->name, $blocksTotal, $blocksFree, $percent);
		$driveEntryList->append(@tmpentry);
	}
	$driveEntryList->signal_connect('select_row', [\&selectDrive, $partEntryList, @allmaps] );

	$driveVbox->pack_start($driveEntryList, 1, 1, 10);

	$driveEntryList->column_titles_show;
	$driveEntryList->show;
}

sub blockCount {
	my $map = shift @_;
	my $free = 0;

	if ($map->disk_order == $NULL) {
		return "-";
	}
	my $i;
	for ($i = $map->disk_order;$i != $NULL; $i = $i->next_by_disk) {
		my $tmpname = $i->dpmeType;
		if (substr($tmpname,0,10) eq "Apple_Free") { 
			$free += $i->dpmePblocks;
		}
	}
	return $free;
}

sub rebootPrompt {
	my $choice;
	
	my $rebootWin = new Gtk::Dialog();
	$rebootWin->set_uposition( 100, 100);
	$rebootWin->signal_connect("destroy", \&destroy_window, \$rebootWin);
	$rebootWin->signal_connect("delete_event", \&destroy_window, \$rebootWin);
	$rebootWin->set_title("Reboot?");
	$rebootWin->show;
	$rebootWin->grab_focus();
	$rebootWin->set_modal(1);

	my $rebootText = "You have saved changes to a partition map.\nYou MUST reboot now for the changes to take effect.\nReboot?";
	my $rebootLabel = new Gtk::Label $rebootText;
	$rebootWin->vbox->pack_start($rebootLabel, 0, 0, 0);
	$rebootLabel->show;

	my $rebootOk = new Gtk::Button "Reboot";
	$rebootOk->signal_connect("clicked", sub {$choice = 1; $rebootWin->destroy;});
	$rebootWin->action_area->pack_start($rebootOk, 1, 1, 0);
	$rebootOk->show;

	my $rebootDont = new Gtk::Button "Don't Reboot";
	$rebootDont->signal_connect("clicked", sub {$choice = 0; $rebootWin->destroy;});
	$rebootWin->action_area->pack_start($rebootDont, 1, 1, 0);
	$rebootDont->show;

	while (! defined $choice) {
		main_iteration Gtk;
	}
	return $choice;
}

sub savePrompt {
	my $map = shift;

	my $choice;

	my $saveWin = new Gtk::Dialog();
	$saveWin->set_uposition( 50, 50);
	$saveWin->signal_connect("destroy", \&destroy_window, \$saveWin);
	$saveWin->signal_connect("delete_event", \&destroy_window, \$saveWin);
	$saveWin->set_title("Save changes?");
	$saveWin->show;
	$saveWin->grab_focus();
	$saveWin->set_modal(1);

	my $saveText = "You have made changes to this partition map. \nDo you wish to save the changes?";
	$saveText .= "\n\nWARNING : SAVING THESE CHANGES WILL DELETE ALL INFORMATION ON THE\nPARTITIONS YOU CHANGED. YOUR MACHINE MAY NOT BOOT AFTER SAVING THE CHANGES.\n";
$saveText .= "\n\n";
$saveText .= "LINUXPPC INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE \n";
$saveText .= "    INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS \n";
$saveText .= "    FOR A PARTICULAR PURPOSE. \n";
$saveText .= "\n";
$saveText .= "    IN NO EVENT SHALL LINUXPPC INC. BE LIABLE FOR ANY SPECIAL, INDIRECT, OR \n";
$saveText .= "    CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM \n";
$saveText .= "    LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT, \n";
$saveText .= "    NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION \n";
$saveText .= "    WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. \n";
$saveText .= "\n";
$saveText .= " In short this is powerful stuff and you have been warned.\n";
$saveText .= "\n";
	my $saveLabel = new Gtk::Label $saveText;
	$saveWin->vbox->pack_start($saveLabel, 0, 0, 0);
	$saveLabel->show;

	my $saveOk = new Gtk::Button "Destructive Save";
	$saveOk->signal_connect("clicked", sub {$choice = 1; $saveWin->destroy;});
	$saveWin->action_area->pack_start($saveOk, 1, 1, 0);
	$saveOk->show;

	my $saveDont = new Gtk::Button "Safely Exit";
	$saveDont->signal_connect("clicked", sub {$choice = 0; $saveWin->destroy;});
	$saveWin->action_area->pack_start($saveDont, 1, 1, 0);
	$saveDont->show;

	while (! defined $choice) {
		main_iteration Gtk;
	}
	return $choice;
}

sub aboutDialog {
	my $choice;

	my $aboutWin = new Gtk::Dialog();
	$aboutWin->set_uposition( 50, 50);
	$aboutWin->signal_connect("destroy", \&destroy_window, \$aboutWin);
	$aboutWin->signal_connect("delete_event", \&destroy_window, \$aboutWin);
	$aboutWin->set_title("About Author");
	$aboutWin->show;
	$aboutWin->grab_focus();
	$aboutWin->set_modal(1);

	my $aboutText = "perldisk.pl - perl/Gtk disk partitioning utility\n";
	$aboutText .= "by Brian Bresen <bbresen\@linuxppc.org>\n";
	$aboutText .= "under the GPL.\n";
	$aboutText .= "\n\n";
	$aboutText .= "LINUXPPC INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE \n";
	$aboutText .= "    INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS \n";
	$aboutText .= "    FOR A PARTICULAR PURPOSE. \n\n";
	$aboutText .= "    IN NO EVENT SHALL LINUXPPC INC. BE LIABLE FOR ANY SPECIAL, INDIRECT, OR \n";
	$aboutText .= "    CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM \n";
	$aboutText .= "    LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT, \n";
	$aboutText .= "    NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION \n";
	$aboutText .= "    WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. \n\n";
	$aboutText .= " In short this is powerful stuff and you have been warned.\n";
	$aboutText .= "\n";
	my $aboutLabel = new Gtk::Label $aboutText;
	$aboutWin->vbox->pack_start($aboutLabel, 0, 0, 0);
	$aboutLabel->show;

	my $aboutOk = new Gtk::Button "OK";
	$aboutOk->signal_connect("clicked", sub {$choice = 1; $aboutWin->destroy;});
	$aboutWin->action_area->pack_start($aboutOk, 1, 1, 0);
	$aboutOk->show;

	while (! defined $choice) {
		main_iteration Gtk;
	}
	return $choice;
}

sub selectDrive {
	my ($drivelist) = shift @_;
	my ($partlist) = shift @_;
#	my ($map) = shift @_;
	my @maps = (@_);

	if (defined ($drivelist->selection())) {
		my $row = $drivelist->selection();

		#Check for change in previous map
		if ($map->checkChange()) {
			if (savePrompt($map) ) {
				$map->write_partition_map();
				$changesmade = 1;
			} else {
				my $i = 0;
				while ($maps[$i] != $map) {
					$i++;
				}
				$maps[$i] = $NULL;
			}
		}

		$map = $maps[$row];

		$partlist->clear;
		if ($map->disk_order == $NULL) {
			$map->read_partition_map();

			my $blocksTotal = $map->sbBlkCount;
			my $blocksFree = blockCount($map);
			my $percent = (($blocksTotal - $blocksFree) / $blocksTotal) * 100;
			$percent = sprintf "%3.1f",$percent;
			$percent .= "%";

			$drivelist->set_text($row,2,$blocksFree);
			$drivelist->set_text($row,3,$percent);
		}
		fillPartData($map,$partlist);

		return $map;
	} else {
		print "Strange happenings select_row signal when nothing selected\n";
		return;
	}
}

sub fillPartData {
	my ($map,$clist) = @_;
	my ($i,$value,$prefix);

	my $count = 1;
	for ($i = $map->disk_order;$i != $NULL; $i = $i->next_by_disk) {
		$value = ($i->dpmePblocks * $map->sbBlkSize);
		($value,$prefix) = $map->adjust_value_and_compute_prefix($value);
		if (($prefix ne " ") && ($prefix ne "K")) { 
			$value = sprintf "%5.1f%s",$value,$prefix;
		} else {
			$value = " "; 
		}

		my @tmparray = ($count, $i->dpmeType, $i->dpmeName, $i->dpmePblocks, $i->dpmePblockStart, $value);

		#now append tmparray to the clist 
		$clist->append(@tmparray);

		$count++;
	}
}

sub Quit {
	my $callingWidget = shift @_;

	#Check for change in previous map
	if ($map->checkChange()) {
		if (savePrompt($map) ) {
			$map->write_partition_map();
			$changesmade = 1;
		}
	}
	if (($changesmade) && (rebootPrompt())) {
		system ("init 6");
	} else {
		Gtk->exit(0);
	}
}

sub addPrompt {
	my $size = shift @_;

	my ($name,$blocks);
	my $type = "Linux_PPC";
	my $value;

	my $addWin = new Gtk::Dialog();
	$addWin->set_uposition( 100, 100);
	$addWin->signal_connect("destroy", \&destroy_window, \$addWin);
	$addWin->signal_connect("delete_event", \&destroy_window, \$addWin);
	$addWin->set_title("Select new partition parameters");
	$addWin->show;
	$addWin->set_modal(1);

	my $addText = "Please choose the size, type, and name \nfor the new partition.";
	my $addLabel = new Gtk::Label $addText;
	$addWin->vbox->pack_start($addLabel, 0, 0, 0);
	$addLabel->show;

	my $adjustment = new Gtk::Adjustment($size, 0, $size, 1.0, 1024.0, 0.0);

	#add dial for blocks
	my $addDial = new Gtk::Dial $adjustment;
	$addDial->set_update_policy(-continuous);
	$addWin->vbox->pack_start($addDial, 0, 0, 0);
	$addDial->show;

	#add label to show current block value
	$value = getBlockString($size);
	my $addBlockLabel = new Gtk::Label($value);
	$addWin->vbox->pack_start($addBlockLabel, 0, 0, 0);
	$addBlockLabel->show;

	#add menu button for type
	# not just yet TODO

	#add combo widget for name
	my $nameEntry = new Gtk::Combo();
	$nameEntry->entry->set_text("/");
	$nameEntry->set_popdown_strings('/','/boot','/home','swap');
	$nameEntry->set_usize(70,0);
	$addWin->vbox->pack_start($nameEntry, 0, 0, 0);
	$nameEntry->show;

	my $addOk = new Gtk::Button "Add";
	$addOk->signal_connect("clicked", sub {
									$blocks = $addDial->get_adjustment->value; 
									$name = $nameEntry->entry->get_text;
									$addWin->destroy;});
	$addWin->action_area->pack_start($addOk, 1, 1, 0);
	$addOk->show;

	my $addDont = new Gtk::Button "Cancel";
	$addDont->signal_connect("clicked", sub {$blocks = 0; 
										$name = "Anything";
										$addWin->destroy;});
	$addWin->action_area->pack_start($addDont, 1, 1, 0);
	$addDont->show;

	while (! defined $name) {
		$value = getBlockString(int($addDial->get_adjustment->value));
		$addBlockLabel->set($value);
		main_iteration Gtk;
	}
	return ($type,$name,int($blocks));
}

sub getBlockString {
	my $blocks = shift;

	my ($value,$prefix);
	($value,$prefix) = partmap::adjust_value_and_compute_prefix($blocks * 512);
	if (($prefix ne " ") && ($prefix ne "K")) { 
		$value = sprintf "%s %5.1f%s",$blocks, $value,$prefix;
	} else {
		$value = $blocks; 
	}
	return $value;
}

sub partAdd {
	my ($button) = shift @_;
	my ($map, $list) = @_;

	my $part;

	if (defined ($list->selection())) {
		 $part = ($list->selection() + 1);
	} else {
		print "You need to select a free partiton first\n";
		return;
	}
	my $entry = $$map->getEntryByNum($part);

	my $myName = $entry->dpmeType;
	if ((substr($myName,0,10)) eq "Apple_Free") {
		my $size = $entry->dpmePblocks;
		my ($type, $name, $count) = addPrompt($size);
		#printf "my type: %s, my name: %s, my size: %s\n",$type,$name,$count;
		if ($count != 0) {
			if ($size == $count) {
				$$map->setPartType($entry,$type);
				$$map->setPartName($entry,$name);
			} else {
				$$map->insertPart($entry,$type,$name,$count);
			}
			$list->clear;
			fillPartData($$map,$list);
		}
	} else {
		print "Hey you need to select a free partiton!\n";
		return;
	}
	return;
	
}

sub partDelete {
	my ($button) = shift @_;
	my ($map,$list) = @_;

	my $type = "Apple_Free";
	my $name = "Extra";
	my $part;
	if (defined ($list->selection())) {
		 $part = ($list->selection() + 1);
	} else {
		return;
	}

	my $entry = $$map->getEntryByNum($part);
	#print "Attempting to change type of partition number $part to $type.\n";
	#print "Attempting to change name of partition number $part to $name.\n";

	$$map->setPartType($entry,$type);
	$$map->setPartName($entry,$name);

	$$map->combineFree($entry);

	#Reshow the partmap
	$list->clear;
	fillPartData($$map,$list);

	return;
}

sub partName {
	my ($button) = shift @_;
	my ($map,$list,$combo) = @_;

	my $name = $combo->get_text;
	my $part;
	if (defined ($list->selection())) {
		 $part = ($list->selection() + 1);
	} else {
		return;
	}
	print "Attempting to change name of partition number $part to $name \n";
	my $entry = $$map->getEntryByNum($part);

	$$map->setPartName($entry,$name);
	$part--;						# This is the row again
	$list->set_text($part,2,$name);
}

sub driveList {
	my @drive;

	#$drive[0] = './my_little_drive';
	#$drive[1] = './ide_drive';

	my (@longlist,@list);
	my ($i,$input);

	open (PARTS, "/proc/partitions");

	while ($input = <PARTS>) {
		chop($input);
		next if ($input =~ /name/);
		next if ($input eq "");
		my @temp = split (/ /, $input);
		push (@longlist,(pop(@temp)));
	}
	close(PARTS);

	for $i (0 .. ($#longlist - 1)) {
		if ((length($longlist[$i]) == 3) && (length($longlist[$i+1]) != 3)) {
			push(@list,( "/dev/" . $longlist[$i]));
		}
	}
	return @list;
	#return @drive;
}

sub partConvert {
	my ($button) = shift @_;
	my ($map,$list) = @_;

	my $part;

	if (defined ($list->selection())) {
		 $part = ($list->selection() + 1);
	} else {
		return;
	}

	my $drive = $$map->name;

	print "$drive$part";
	Gtk->exit(0);
}
